@@ -451,6 +451,19 @@ static double exact_cpu_count(const char *cg)
 	return rv;
 }
 
+/*
+ * Return true if cfs quota of the cgroup is neg / not set
+ */
+static bool cfs_quota_disabled(const char *cg)
+{
+	int64_t cfs_quota;
+
+	if (!read_cpu_cfs_param(cg, "quota", &cfs_quota))
+		return true;
+
+	return cfs_quota < 0;
+}
+
 /*
  * Return the maximum number of visible CPUs based on CPU quotas.
  * If there is no quota set, zero is returned.
@@ -670,9 +683,9 @@ int cpuview_proc_stat(const char *cg, const char *cpuset,
 		}
 
 		if (user_surplus > 0)
-			lxcfs_debug("leftover user: %lu for %s\n", user_surplus, cg);
+			lxcfs_debug("leftover user: %" PRIu64 "for %s\n", user_surplus, cg);
 		if (system_surplus > 0)
-			lxcfs_debug("leftover system: %lu for %s\n", system_surplus, cg);
+			lxcfs_debug("leftover system: %" PRIu64 "for %s\n", system_surplus, cg);
 
 		for (curcpu = 0, i = -1; curcpu < nprocs; curcpu++) {
 			if (!stat_node->usage[curcpu].online)
@@ -687,10 +700,6 @@ int cpuview_proc_stat(const char *cg, const char *cpuset,
 			stat_node->view[curcpu].system 	+= diff[curcpu].system;
 			stat_node->view[curcpu].idle 	+= diff[curcpu].idle;
 
-			user_sum 	+= stat_node->view[curcpu].user;
-			system_sum 	+= stat_node->view[curcpu].system;
-			idle_sum 	+= stat_node->view[curcpu].idle;
-
 			diff_user 	+= diff[curcpu].user;
 			diff_system 	+= diff[curcpu].system;
 			diff_idle 	+= diff[curcpu].idle;
@@ -699,31 +708,39 @@ int cpuview_proc_stat(const char *cg, const char *cpuset,
 				max_diff_idle_index 	= curcpu;
 			}
 
-			lxcfs_v("curcpu: %d, diff_user: %lu, diff_system: %lu, diff_idle: %lu\n", curcpu, diff[curcpu].user, diff[curcpu].system, diff[curcpu].idle);
+			lxcfs_v("curcpu: %d, diff_user: %" PRIu64 ", diff_system: %" PRIu64 ", diff_idle: %" PRIu64 "\n", curcpu, diff[curcpu].user, diff[curcpu].system, diff[curcpu].idle);
+		}
+		lxcfs_v("total. diff_user: %" PRIu64 ", diff_system: %" PRIu64 ", diff_idle: %" PRIu64 "\n", diff_user, diff_system, diff_idle);
+
+		for (curcpu = 0; curcpu < nprocs; curcpu++) {
+			user_sum 	+= stat_node->view[curcpu].user;
+			system_sum 	+= stat_node->view[curcpu].system;
+			idle_sum 	+= stat_node->view[curcpu].idle;
 		}
-		lxcfs_v("total. diff_user: %lu, diff_system: %lu, diff_idle: %lu\n", diff_user, diff_system, diff_idle);
 
 		/* revise cpu usage view to support partial cpu case. */
 		exact_cpus = exact_cpu_count(cg);
-		if (exact_cpus < (double)max_cpus){
+
+		/* skip revise cpu when cfs quota is disabled (exact_cpus == 0) */
+		if (!cfs_quota_disabled(cg) && exact_cpus < (double)max_cpus){
 			uint64_t delta = (uint64_t)((double)(diff_user + diff_system + diff_idle) * (1 - exact_cpus / (double)max_cpus));
 
 			lxcfs_v("revising cpu usage view to match the exact cpu count [%f]\n", exact_cpus);
-			lxcfs_v("delta: %lu\n", delta);
-			lxcfs_v("idle_sum before: %lu\n", idle_sum);
+			lxcfs_v("delta: %" PRIu64 "\n", delta);
+			lxcfs_v("idle_sum before: %" PRIu64 "\n", idle_sum);
 			if (idle_sum > delta)
 				idle_sum = idle_sum - delta;
 			else
 				idle_sum = 0;
-			lxcfs_v("idle_sum after: %lu\n", idle_sum);
+			lxcfs_v("idle_sum after: %l" PRIu64 "\n", idle_sum);
 
 			curcpu = max_diff_idle_index;
-			lxcfs_v("curcpu: %d, idle before: %lu\n", curcpu, stat_node->view[curcpu].idle);
+			lxcfs_v("curcpu: %d, idle before: %" PRIu64 "\n", curcpu, stat_node->view[curcpu].idle);
 			if (stat_node->view[curcpu].idle > delta)
 				stat_node->view[curcpu].idle = stat_node->view[curcpu].idle - delta;
 			else
 				stat_node->view[curcpu].idle = 0;
-			lxcfs_v("curcpu: %d, idle after: %lu\n", curcpu, stat_node->view[curcpu].idle);
+			lxcfs_v("curcpu: %d, idle after: %" PRIu64 "\n", curcpu, stat_node->view[curcpu].idle);
 		}
 	} else {
 		for (curcpu = 0; curcpu < nprocs; curcpu++) {
@@ -761,21 +778,50 @@ int cpuview_proc_stat(const char *cg, const char *cpuset,
 	buf_size -= l;
 	total_len += l;
 
-	/* Render visible CPUs */
-	for (curcpu = 0, i = -1; curcpu < nprocs; curcpu++) {
-		if (!stat_node->usage[curcpu].online)
-			continue;
+	/* Render visible CPUs 
+	Assume there are K CPUs: 0, 1, 2, ..., K-1.
+	Among them, there are M online CPUs with index: a1, a2, ... aN ... aM (M >= N)
+	N = max_cpus, M = number of online CPUs
 
+	There will be N rendered cpus, indexed from 0 to N-1, cpu times of the cpus are calculated from those formula:
+	- user_time[0] = stat_node->view[0].user + stat_node->view[1].user + ... + stat_node->view[a1].user
+	- user_time[1] = stat_node->view[a1+1].user + stat_node->view[a1+1].user + ... + stat_node->view[a2].user
+	...
+	- user_time[N-2] = stat_node->view[a(N-2)+1].user + stat_node->view[a(N-2)+2].user + ... 
+	                    + stat_node->view[a(N-1)].user
+	- user_time[N-1] = stat_node->view[a(N-1)+1].user + stat_node->view[a(N-1)+2].user + ...
+	                    + stat_node->view[aN] + ... + stat_node->view[K-1] (sum of all remaining CPUs)
+	
+	Similar formula applied for system and idle time
+	*/
+
+	uint64_t curcpu_view_user_sum = 0, curcpu_view_system_sum = 0, curcpu_view_idle_sum = 0;
+	for (curcpu = 0, i = -1; curcpu < nprocs; curcpu++) {
+		curcpu_view_user_sum += stat_node->view[curcpu].user;
+		curcpu_view_system_sum += stat_node->view[curcpu].system;
+		curcpu_view_idle_sum += stat_node->view[curcpu].idle;
+
+		if (!stat_node->usage[curcpu].online && curcpu < nprocs - 1) {
+			continue;
+		}
+		
 		i++;
 
-		if (max_cpus > 0 && i == max_cpus)
-			break;
+		if (max_cpus > 0 && i >= max_cpus) {
+			// max(i) = count(rendered cpus) = max_cpus - 1
+			i--;
+		}
+
+		if (max_cpus > 0 && i == max_cpus - 1 && curcpu < nprocs - 1) {
+			// last 'rendered' cpu, sum until reaches the last cpu
+			continue;
+		}
 
 		l = snprintf(buf, buf_size, "cpu%d %" PRIu64 " 0 %" PRIu64 " %" PRIu64 " 0 0 0 0 0 0\n",
 			     i,
-			     stat_node->view[curcpu].user,
-			     stat_node->view[curcpu].system,
-			     stat_node->view[curcpu].idle);
+			     curcpu_view_user_sum,
+			     curcpu_view_system_sum,
+			     curcpu_view_idle_sum);
 		lxcfs_v("cpu: %s\n", buf);
 		if (l < 0) {
 			lxcfs_error("Failed to write cache");
@@ -791,6 +837,10 @@ int cpuview_proc_stat(const char *cg, const char *cpuset,
 		buf += l;
 		buf_size -= l;
 		total_len += l;
+
+		curcpu_view_user_sum = 0;
+		curcpu_view_system_sum = 0;
+		curcpu_view_idle_sum = 0;
 	}
 
 	/* Pass the rest of /proc/stat, start with the last line read */
