@@ -299,6 +299,43 @@ static inline bool startswith(const char *line, const char *pref)
 	return strncmp(line, pref, strlen(pref)) == 0;
 }
 
+static void get_swap_info(const char *cgroup, uint64_t memlimit,
+			  uint64_t memusage, uint64_t *swtotal,
+			  uint64_t *swusage, uint64_t *memswpriority)
+{
+	__do_free char *memswusage_str = NULL, *memswpriority_str = NULL;
+	uint64_t memswlimit = 0, memswusage = 0;
+	int ret;
+
+	*swtotal = *swusage = 0;
+	*memswpriority = 1;
+
+	memswlimit = get_min_memlimit(cgroup, true);
+	if (memswlimit > 0) {
+		ret = cgroup_ops->get_memory_swap_current(cgroup_ops, cgroup, &memswusage_str);
+		if (ret < 0 || safe_uint64(memswusage_str, &memswusage, 10) != 0)
+			return;
+
+		if (liblxcfs_memory_is_cgroupv2()) {
+			*swtotal = memswlimit / 1024;
+			*swusage = memswusage / 1024;
+		} else {
+			if (memlimit > memswlimit)
+				*swtotal = 0;
+			else
+				*swtotal = (memswlimit - memlimit) / 1024;
+			if (memusage > memswusage || swtotal == 0)
+				*swusage = 0;
+			else
+				*swusage = (memswusage - memusage) / 1024;
+		}
+
+		ret = cgroup_ops->get_memory_swappiness(cgroup_ops, cgroup, &memswpriority_str);
+		if (ret >= 0)
+			safe_uint64(memswpriority_str, memswpriority, 10);
+	}
+}
+
 static int proc_swaps_read(char *buf, size_t size, off_t offset,
 			   struct fuse_file_info *fi)
 {
@@ -307,7 +344,7 @@ static int proc_swaps_read(char *buf, size_t size, off_t offset,
 	struct fuse_context *fc = fuse_get_context();
 	bool wants_swap = lxcfs_has_opt(fuse_get_context()->private_data, LXCFS_SWAP_ON);
 	struct file_info *d = INTTYPE_TO_PTR(fi->fh);
-	uint64_t memswlimit = 0, memlimit = 0, memusage = 0, memswusage = 0,
+	uint64_t memlimit = 0, memusage = 0,
 		 swtotal = 0, swusage = 0, memswpriority = 1,
 		 hostswtotal = 0, hostswfree = 0;
 	ssize_t total_len = 0;
@@ -353,26 +390,8 @@ static int proc_swaps_read(char *buf, size_t size, off_t offset,
 	if (safe_uint64(memusage_str, &memusage, 10) < 0)
 		lxcfs_error("Failed to convert memusage %s", memusage_str);
 
-	if (wants_swap) {
-		memswlimit = get_min_memlimit(cgroup, true);
-		if (memswlimit > 0) {
-			ret = cgroup_ops->get_memory_swap_current(cgroup_ops, cgroup, &memswusage_str);
-			if (ret >= 0 && safe_uint64(memswusage_str, &memswusage, 10) == 0) {
-				if (memlimit > memswlimit)
-					swtotal = 0;
-				else
-					swtotal = (memswlimit - memlimit) / 1024;
-				if (memusage > memswusage || swtotal == 0)
-					swusage = 0;
-				else
-					swusage = (memswusage - memusage) / 1024;
-			}
-
-			ret = cgroup_ops->get_memory_swappiness(cgroup_ops, cgroup, &memswpriority_str);
-			if (ret >= 0)
-				safe_uint64(memswpriority_str, &memswpriority, 10);
-		}
-	}
+	if (wants_swap)
+		get_swap_info(cgroup, memlimit, memusage, &swtotal, &swusage, &memswpriority);
 
 	total_len = snprintf(d->buf, d->size, "Filename\t\t\t\tType\t\tSize\tUsed\tPriority\n");
 
@@ -438,13 +457,13 @@ static void get_blkio_io_value(char *str, unsigned major, unsigned minor,
 	len = strlen(key);
 	while (*str) {
 		if (startswith(str, key)) {
-			sscanf(str + len, "%lu", v);
+			sscanf(str + len, "%" PRIu64, v);
 			return;
 		}
 		eol = strchr(str, '\n');
 		if (!eol)
 			return;
-		str = eol+1;
+		str = eol + 1;
 	}
 }
 
@@ -598,7 +617,26 @@ static int proc_diskstats_read(char *buf, size_t size, off_t offset,
 		    stats.read_sectors || stats.write_sectors || stats.read_ticks ||
 		    stats.write_ticks || stats.ios_pgr || stats.total_ticks || stats.rq_ticks ||
 		    stats.discard_merged || stats.discard_sectors || stats.discard_ticks)
-			snprintf(lbuf, 256, "%u       %u %s %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
+			snprintf(
+				lbuf,
+				256,
+				"%u       %u" /* major, minor */
+				" %s"         /* dev_name */
+				" %" PRIu64   /* read */
+				" %" PRIu64   /* read_merged */
+				" %" PRIu64   /* read_sectors */
+				" %" PRIu64   /* read_ticks */
+				" %" PRIu64   /* write */
+				" %" PRIu64   /* write_merged */
+				" %" PRIu64   /* write_sectors */
+				" %" PRIu64   /* write_ticks */
+				" %" PRIu64   /* ios_pgr */
+				" %" PRIu64   /* total_ticks */
+				" %" PRIu64   /* rq_ticks */
+				" %" PRIu64   /* discard_merged */
+				" %" PRIu64   /* discard_sectors */
+				" %" PRIu64   /* discard_ticks */
+				"\n",
 				stats.major,
 				stats.minor,
 				stats.dev_name,
@@ -956,7 +994,19 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 
 		curcpu++;
 
-		ret = sscanf(line, "%*s %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu",
+		ret = sscanf(
+			   line,
+			   "%*s"        /* <skip> */
+			   " %" PRIu64  /* user */
+			   " %" PRIu64  /* nice */
+			   " %" PRIu64  /* system */
+			   " %" PRIu64  /* idle */
+			   " %" PRIu64  /* iowait */
+			   " %" PRIu64  /* irq */
+			   " %" PRIu64  /* softirq */
+			   " %" PRIu64  /* steal */
+			   " %" PRIu64  /* guest */
+			   " %" PRIu64, /* guest_nice */
 			   &user,
 			   &nice,
 			   &system,
@@ -1033,7 +1083,21 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 
 	cache = d->buf;
 
-	int cpuall_len = snprintf(cpuall, CPUALL_MAX_SIZE, "cpu  %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
+	int cpuall_len = snprintf(
+			cpuall,
+			CPUALL_MAX_SIZE,
+			"cpu  "
+			" %" PRIu64 /* user_sum */
+			" %" PRIu64 /* nice_sum */
+			" %" PRIu64 /* system_sum */
+			" %" PRIu64 /* idle_sum */
+			" %" PRIu64 /* iowait_sum */
+			" %" PRIu64 /* irq_sum */
+			" %" PRIu64 /* softirq_sum */
+			" %" PRIu64 /* steal_sum */
+			" %" PRIu64 /* guest_sum */
+			" %" PRIu64 /* guest_nice_sum */
+			"\n",
 			user_sum,
 			nice_sum,
 			system_sum,
@@ -1141,7 +1205,7 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 	struct fuse_context *fc = fuse_get_context();
 	bool wants_swap = lxcfs_has_opt(fuse_get_context()->private_data, LXCFS_SWAP_ON);
 	struct file_info *d = INTTYPE_TO_PTR(fi->fh);
-	uint64_t memlimit = 0, memusage = 0, memswlimit = 0, memswusage = 0,
+	uint64_t memlimit = 0, memusage = 0,
 		 hosttotal = 0, swfree = 0, swusage = 0, swtotal = 0,
 		 memswpriority = 1;
 	struct memory_stat mstat = {};
@@ -1193,26 +1257,8 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 	 * Following values are allowed to fail, because swapaccount might be
 	 * turned off for current kernel.
 	 */
-	if (wants_swap) {
-		memswlimit = get_min_memlimit(cgroup, true);
-		if (memswlimit > 0) {
-			ret = cgroup_ops->get_memory_swap_current(cgroup_ops, cgroup, &memswusage_str);
-			if (ret >= 0 && safe_uint64(memswusage_str, &memswusage, 10) == 0) {
-				if (memlimit > memswlimit)
-					swtotal = 0;
-				else
-					swtotal = (memswlimit - memlimit) / 1024;
-				if (memusage > memswusage || swtotal == 0)
-					swusage = 0;
-				else
-					swusage = (memswusage - memusage) / 1024;
-			}
-		}
-
-		ret = cgroup_ops->get_memory_swappiness(cgroup_ops, cgroup, &memswpriority_str);
-		if (ret >= 0)
-			safe_uint64(memswpriority_str, &memswpriority, 10);
-	}
+	if (wants_swap)
+		get_swap_info(cgroup, memlimit, memusage, &swtotal, &swusage, &memswpriority);
 
 	f = fopen_cached("/proc/meminfo", "re", &fopen_cache);
 	if (!f)
@@ -1251,7 +1297,9 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 				   This is because the kernel can swap as much as it
 				   wants and not only up to swtotal. */
 
-				swtotal = memlimit + swtotal;
+				if (!liblxcfs_memory_is_cgroupv2())
+					swtotal += memlimit;
+
 				if (hostswtotal < swtotal) {
 					swtotal = hostswtotal;
 				}
