@@ -1,28 +1,11 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
-
 #include "config.h"
 
-#ifdef HAVE_FUSE3
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 30
-#endif
-#else
-#ifndef FUSE_USE_VERSION
-#define FUSE_USE_VERSION 26
-#endif
-#endif
-
-#define _FILE_OFFSET_BITS 64
-
-#define __STDC_FORMAT_MACROS
+#include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <fuse.h>
 #include <inttypes.h>
 #include <libgen.h>
 #include <pthread.h>
@@ -46,11 +29,12 @@
 #include <sys/sysinfo.h>
 #include <sys/vfs.h>
 
+#include "sysfs_fuse.h"
+
 #include "bindings.h"
 #include "memory_utils.h"
 #include "cgroups/cgroup.h"
 #include "lxcfs_fuse_compat.h"
-#include "sysfs_fuse.h"
 #include "utils.h"
 
 static int sys_devices_system_cpu_online_read(char *buf, size_t size,
@@ -69,7 +53,7 @@ static int sys_devices_system_cpu_online_read(char *buf, size_t size,
 	ssize_t total_len = 0;
 
 	if (offset) {
-		int left;
+		size_t left;
 
 		if (!d->cached)
 			return 0;
@@ -118,7 +102,7 @@ static int sys_devices_system_cpu_online_read(char *buf, size_t size,
 	d->size = (int)total_len;
 	d->cached = 1;
 
-	if (total_len > size)
+	if ((size_t)total_len > size)
 		total_len = size;
 
 	memcpy(buf, d->buf, total_len);
@@ -178,7 +162,7 @@ __lxcfs_fuse_ops int sys_getattr(const char *path, struct stat *sb)
 	}
 
 	if (strcmp(path, "/sys/devices/system/cpu/online") == 0) {
-		sb->st_size = 4096;
+		sb->st_size = get_sysfile_size (path);
 		sb->st_mode = S_IFREG | 00444;
 		sb->st_nlink = 1;
 		return 0;
